-- require("defines")
local get_location = require("position")

local cfg_take_screenshot = false
local cfg_screenshot_players = false
local cfg_screenshot_chunks = true
local cfg_num_of_ticks_to_take_screenshot = 10
local cfg_screenshot_at_day = true
local cfg_screenshot_with_entity_info = false
local cfg_dump_info = false
local cfg_dump_radius = 100
local cfg_num_of_ticks_to_dump_info = 1

local cfg_positions = {}
--local cfg_position = {-39.5, 15.5}
--local cfg_position = {262.5, -391.5}
--local cfg_position = {-62.5, 48.5}
cfg_positions = {{-61, 170}, {45, 64}, {47, -54}}

local cfg_name = "31921_01_des"


local CAPTURE_WIDTH= 32 * 32 * 0.5 --1920
local CAPTURE_HEIGHT= 32 * 32 * 0.5 -- 1080
local SCREENSHOT_SUFFIX = cfg_name --"_1c_ref"
local REPLACEMENT_PLAYER_NAME = "nil"

script.on_configuration_changed(
function()
	global.posila = global.posila or {}
  global.posila.scr_time_to_live = global.posila.scr_time_to_live or cfg_num_of_ticks_to_take_screenshot
  global.posila.dump_time_to_live = global.posila.dump_time_to_live or cfg_num_of_ticks_to_dump_info
end)

script.on_init(
function()
  global.posila = global.posila or {}
  global.posila.scr_time_to_live = global.posila.scr_time_to_live or cfg_num_of_ticks_to_take_screenshot
  global.posila.dump_time_to_live = global.posila.dump_time_to_live or cfg_num_of_ticks_to_dump_info
end)

script.on_event(defines.events.on_built_entity, 
function(event)

end)

local function print_all(str)
  for k,this_player in pairs(game.players) do
    this_player.print(str)
  end
end

local function function_take_screenshot(this_player, is_real_player)
        if cfg_screenshot_at_day then
          game.surfaces[1].daytime = 0
        end
        
        -- local screenshot_location = {262.5, -391.5} --{-39.5, 15.5} -- get_location(this_player)
        -- local screenshot_location = cfg_position -- {-39.5, 15.5}
        local screenshot_location = this_player.position
        
        game.take_screenshot{position=screenshot_location,
                         resolution={CAPTURE_WIDTH, CAPTURE_HEIGHT},
                         zoom=1,
                         path="MichalTest_1.0.0/"..string.format("%04d_" .. this_player.name .. SCREENSHOT_SUFFIX .. ".png", game.tick),
                         player=(is_real_player and this_player) or nil,
                         show_gui=false,
                         show_entity_info=cfg_screenshot_with_entity_info}
        local p = screenshot_location
        if p.x == nil then
            print_all("Screenshot: position={" .. p[1] .. ", " .. p[2] .. "}, tick=" .. game.tick .. ", player=" .. this_player.name)
        else
            print_all("Screenshot: position={" .. p.x .. ", " .. p.y .. "}, tick=" .. game.tick .. ", player=" .. this_player.name)
        end
end

local function do_take_screenshot(event)
    
    if not global.posila.screenshot_taken then
      print_all("take screenshot")
      for k,pos in pairs(cfg_positions) do
        function_take_screenshot({name = string.format("pos_x%g_y%g", pos.x or pos[1], pos.y or pos[2]), position=pos }, false)
      end
      if cfg_screenshot_players then
        for k,this_player in pairs(game.players) do
          function_take_screenshot(this_player, true)
        end
      end
      
      --local this_player = { name = "nil", position = cfg_position, is_null_player = true } --game.players["sulkiercloud218"]
      --if this_player then
      --for k,this_player in pairs(game.players) do
        -- local this_player = game.players[1]
        
        -- this_player.update_selected_entity({-45.5, -70.5})
        
      if global.posila.scr_time_to_live <= 1 then
        global.posila.screenshot_taken = true
      else
        global.posila.scr_time_to_live = global.posila.scr_time_to_live - 1
      end
    end    
end

local function do_screenshot_chunks(event)    
    if not global.posila.chunks_screenshot_taken then
      print_all("screenshot chunks")
      if cfg_screenshot_at_day then
        game.surfaces[1].daytime = 0
      end
      
      for chunk in game.surfaces[1].get_chunks() do
         local chunk_name = string.format("%s/chunk_x%d_y%d.bmp", cfg_name, chunk.x, chunk.y)
         
         local screenshot_location = { chunk.x * 32 + 512, chunk.y * 32 + 512 }
         
         game.take_screenshot{position=screenshot_location,
                              resolution={1024, 1024},
                              zoom=1,
                              path="MichalTest_1.0.0/".. chunk_name,
                              player=nil,
                              show_gui=false,
                              show_entity_info=cfg_screenshot_with_entity_info}
            
      end
      

        
      global.posila.chunks_screenshot_taken = true
    end    
end

local function make_string_builder()
  -- TODO: learn about metatables
  local sb = { buffer = "" }
  sb.append_line = function(line)
    sb.buffer = sb.buffer .. line .. "\n"
    return sb
  end
  sb.append_format_line = function(fmt, ...)
    return sb.append_line(string.format(fmt, ...))
  end
  return sb
end

function export_entity_inventories(entity, formated_name, output)
  local outputted_header = false
  if entity.has_items_inside() then
    for k = 1,8 do
      local inventory = entity.get_inventory(k)
      if inventory and not inventory.is_empty() then
        for item, count in pairs(inventory.get_contents()) do
          if not outputted_header then
            outputted_header = true
            output.append_line(formated_name .. ":")  
          end
          output.append_format_line("  [%d] \"%s\" = %du;", k, item, count)
        end
      end
    end     
  end
end

function export_inserter(entity, formated_name, output)
  if entity.type == "inserter" and entity.held_stack and entity.held_stack.valid then
    output.append_format_line("%s: [%g, %g] \"%s\" = %du;", 
      formated_name, 
      entity.held_stack_position.x, 
      entity.held_stack_position.y, 
      (entity.held_stack.valid_for_read and entity.held_stack.name) or "nil", 
      (entity.held_stack.valid_for_read and entity.held_stack.count) or 0
    )
  end
end

local transportbelt_connectable = 
{ 
  ["transport-belt"] = 2,
  ["loader"] = 2,
  ["underground-belt"] = 4,
  ["splitter"] = 8
}

function export_transport_belts(entity, formated_name, output)
  local x = transportbelt_connectable[entity.type]
  if x then
    output.append_line(formated_name)
    for i = 1,x do
      local tln = entity.get_transport_line(i)
      for k,v in pairs(tln.get_contents()) do
        output.append_format_line("  [%d] %s = %d", i, k, v)
      end
    end
  end
end

local function dump_info(pos)
  local radius = cfg_dump_radius
  local output = make_string_builder()
  local area = {{pos[1] - radius, pos[2] - radius}, {pos[1] + radius, pos[2] + radius}}
  --local area = {{}}
  output.append_format_line("------------- %d @ X=%g, Y=%g", game.tick, pos[1], pos[2])
  for k,e in pairs(game.surfaces[1].find_entities(area)) do
    --if e.force == game.forces.player then
    --  e.active = false
    --end
    local name = string.format("%s_x%gy%g", e.name, e.position.x, e.position.y)
    export_entity_inventories(e, name, output)
    export_inserter(e, name, output)
    export_transport_belts(e, name, output)
  end
  game.write_file("dump_" .. cfg_name .. ".txt", output.buffer, true)
end


local function do_dump_info(event)
  if global.posila.dumped_info then
    return
  end
  --local pos = {262.5, -391.5} 
  
  local pos = cfg_position
  for k, pos in pairs(cfg_positions) do
    dump_info(pos)
  end 
  
  if global.posila.dump_time_to_live <= 1 then
    global.posila.dumped_info = true
  else
    global.posila.dump_time_to_live = global.posila.dump_time_to_live - 1
  end
end

local function on_tick(event)
  if cfg_dump_info then
    do_dump_info(event)
  end
  
  if cfg_take_screenshot then
    do_take_screenshot(event)
  end
  
  if cfg_screenshot_chunks then
    do_screenshot_chunks(event)
  end
end

if cfg_dump_info or cfg_take_screenshot or cfg_screenshot_chunks then
  script.on_event(defines.events.on_tick, on_tick)
end